function love.load()
	
	boop = love.audio.newSource("boop.wav", "static")
	bip = love.audio.newSource("bip.wav", "static")
	
	TileSize = 32
	TileSpace = TileSize + 1
	TileX = 16
	TileY = 15

	Grid = {}
	for x = -1, TileX + 4 do 
		Grid[x] = {}
		for y = -1, TileY + 1 do
			Grid[x][y] = false
		end
	end
	
	LastShape = 0
	newPiece()
	
	keydown = {}
	keydown["up"] = false
	keydown["down"] = false
	keydown["left"] = false
	keydown["right"] = false
	
	TimeTillDrop = 10.0
	TimeSinceMove = 0.0
	TimeSinceHit = 0.0
	Multiplier = 1
	Score = 0
	
	GameState = "menu"
end



function resetGame()
	keydown["up"] = false
	keydown["down"] = false
	keydown["left"] = false
	keydown["right"] = false
	
	TimeTillDrop = 10.0
	TimeSinceMove = 0.0
	TimeSinceHit = 0.0
	Multiplier = 1
	Score = 0
	
	LastShape = 0
	newPiece()
	
	for x = -1, TileX + 4 do 
		for y = -1, TileY+1 do
			Grid[x][y] = false
		end
	end
	
	for x = 1, TileX + 3 do
		Grid[x][0] = true
		Grid[x][TileY+1] = true
	end
	
	for y = 0, TileY + 1 do
		Grid[0][y] = true
		Grid[TileX+4][y] = true
	end

	Grid[17][1] = true
	Grid[18][1] = true
	Grid[19][1] = true
	Grid[17][15] = true
	Grid[18][15] = true
	Grid[19][15] = true
	
	love.audio.stop(bip)
	love.audio.play(bip)
	
end



function love.update(dt)

	if GameState == "menu" then
	elseif GameState == "gameon" then
	
		TimeTillDrop = TimeTillDrop - dt
		TimeSinceHit = TimeSinceHit + dt
		TimeSinceMove = TimeSinceMove + dt
		Multiplier = math.max(10 - math.floor(TimeSinceHit), 1)
		
		if TimeSinceMove < 0.1 then return end
		TimeSinceMove = 0.0	

		if keydown["up"]	and canMove(0,-1) then PieceY = PieceY - 1 end
		if keydown["down"] 	and canMove(0, 1) then PieceY = PieceY + 1 end
		if keydown["left"] 	and canMove(-1,0) then PieceX = PieceX - 1 end
		if keydown["right"] and canMove( 1,0) then PieceX = PieceX + 1 end

		if TimeTillDrop < 0.0 then 
			if (PieceX <= 13) then 
				drop() 
			else
				love.audio.stop(boop)
				love.audio.play(boop)
				GameState = "gameover"
			end
		end
	
	elseif GameState == "gameover" then
	end

end



function love.draw()
	if GameState == "menu" then --************************
	
		love.graphics.print("LÖVE-Squarez\n\nA LÖVE implementation of the flashgame Squarez\n" ..
		"Original implementation by Brian Cable and concept by Adam Pedersen\n\n" ..
		"This implementation by Rock Hardbuns\n\n" ..
		"Place the bricks into 3x3 squares to make them go away\n" ..
		"Use the arrow keys to move the brick. V and B to rotate it left and right\n" ..
		"and SPACE to drop it. Also hit Q if you want to start over.\n\n" ..
		"The game ends when you can not move any more bricks onto the board.\n\n" ..
		"Hit any key to begin.", 40, 40)
	
	elseif GameState == "gameon" then --************************
	
		love.graphics.setColor( 255, 255, 255, 255 )
		love.graphics.print("Score: " .. Score, 5, 5)
		love.graphics.print("Multiplier: " .. Multiplier, 250, 5)
		love.graphics.print("Drop in... " .. math.floor(TimeTillDrop), 550, 5)
		love.graphics.rectangle( "line", TileSize, TileSize, TileSpace * TileX + 1, TileSpace * TileY + 1)
				
		--Stage
		love.graphics.rectangle( "line", (TileX + 1) * TileSpace, 2 * TileSpace, TileSpace * 3, TileSpace * 13 )
		
		love.graphics.setColor( 0, 0, 0, 255 )
		love.graphics.line( (TileX + 1) * TileSpace, 2 * TileSpace, (TileX + 1) * TileSpace, TileSpace * 15)
			
		--Field	
		for x = 1, TileX do
			for y = 1, TileY do
				if Grid[x][y] == true then
					love.graphics.setColor( 55, 55, 55, 255 )
					love.graphics.rectangle( "fill", x * TileSpace, y * TileSpace, TileSize, TileSize )
					love.graphics.setColor( 255, 255, 255, 255 )
					love.graphics.rectangle( "line", x * TileSpace, y * TileSpace, TileSize, TileSize )
				end
			end
		end

		--Piece
		for x = 1, 3 do
			for y = 1, 3 do
				if Piece[x][y] == true then
					love.graphics.setColor( 155, 55, 55, 255 )
					love.graphics.rectangle( "fill", (PieceX + x) * TileSpace, (PieceY + y) * TileSpace, TileSize, TileSize )
					love.graphics.setColor( 255, 255, 255, 255 )
					love.graphics.rectangle( "line", (PieceX + x) * TileSpace, (PieceY + y) * TileSpace, TileSize, TileSize )
				end
			end
		end
	
	elseif GameState == "gameover" then --************************
		love.graphics.print("Game over\nFinal score:" .. Score, 40, 40)
	end
	


end



function love.mousepressed(x, y, button)
	if GameState == "menu" then
		resetGame()
		GameState = "gameon"
	elseif GameState == "gameon" then
	elseif GameState == "gameover" then
		GameState = "menu"
	end
end



function love.keypressed(key, unicode)

	if GameState == "menu" then
		resetGame()
		GameState = "gameon"
	elseif GameState == "gameon" then
	
		if 		key == "up" 	then keydown["up"] 		= true 
		elseif 	key == "down" 	then keydown["down"] 	= true 
		elseif 	key == "left" 	then keydown["left"] 	= true 
		elseif 	key == "right" 	then keydown["right"]	= true
		end 
		
		if 		key == "v"	and canRotate(1) then rotate(1)
		elseif 	key == "b"	and canRotate(2) then rotate(2)
		elseif 	key == "q"	then GameState = "gameover"
		elseif 	key == " "	then drop() end
	
	elseif GameState == "gameover" then
		GameState = "menu"
	end

	
end



function love.keyreleased(key, unicode)

	if 		key == "up" 	then keydown["up"] 		= false 
	elseif 	key == "down" 	then keydown["down"] 	= false 
	elseif 	key == "left" 	then keydown["left"] 	= false 
	elseif 	key == "right" 	then keydown["right"] 	= false
	end 

end



function drop()
		if 	not(Piece[3][1] or Piece[3][2] or Piece[3][3])then --small piece
			if (PieceX > 14) then return end
		else
			if (PieceX > 13) then return end --big piece
		end
		
		for x = 1, 3 do
			for y = 1, 3 do
				Grid[PieceX+x][PieceY+y] = Piece[x][y] or Grid[PieceX+x][PieceY+y]
			end
		end
		TimeTillDrop = 10.0
		hitScan(PieceX+1, PieceY+1)
		newPiece()
end



function love.focus(f)
end



function love.quit()
	print("Goodbye!")
end



function canRotate(dir)
	if LastShape < 3 then return false end
	
	tmpPiece = { {false,false,false}, {false,false,false}, {false,false,false} }
	
	if dir == 1 then
		tmpPiece[1][3] = Piece[1][1]
		tmpPiece[1][2] = Piece[2][1]
		tmpPiece[1][1] = Piece[3][1]
		tmpPiece[2][1] = Piece[3][2]
		tmpPiece[3][1] = Piece[3][3]
		tmpPiece[3][2] = Piece[2][3]
		tmpPiece[3][3] = Piece[1][3]
		tmpPiece[2][3] = Piece[1][2]
	else
		tmpPiece[1][1] = Piece[1][3]
		tmpPiece[2][1] = Piece[1][2]
		tmpPiece[3][1] = Piece[1][1]
		tmpPiece[3][2] = Piece[2][1]
		tmpPiece[3][3] = Piece[3][1]
		tmpPiece[2][3] = Piece[3][2]
		tmpPiece[1][3] = Piece[3][3]
		tmpPiece[1][2] = Piece[2][3]
	end
	
	tmpPiece[2][2] = Piece[2][2]
	
	local R = true
	for x = 1, 3 do
		for y = 1, 3 do
			R = R and not(( Grid[PieceX+x][PieceY+y] and tmpPiece[x][y]) == true)
			if R == false then return false end
		end
	end
	
	return R
	
end



function rotate(dir)
	if LastShape < 3 then return end
	tmpPiece = { {false,false,false}, {false,false,false}, {false,false,false} }
	
	if dir == 1 then
		tmpPiece[1][3] = Piece[1][1]
		tmpPiece[1][2] = Piece[2][1]
		tmpPiece[1][1] = Piece[3][1]
		tmpPiece[2][1] = Piece[3][2]
		tmpPiece[3][1] = Piece[3][3]
		tmpPiece[3][2] = Piece[2][3]
		tmpPiece[3][3] = Piece[1][3]
		tmpPiece[2][3] = Piece[1][2]
	else
		tmpPiece[1][1] = Piece[1][3]
		tmpPiece[2][1] = Piece[1][2]
		tmpPiece[3][1] = Piece[1][1]
		tmpPiece[3][2] = Piece[2][1]
		tmpPiece[3][3] = Piece[3][1]
		tmpPiece[2][3] = Piece[3][2]
		tmpPiece[1][3] = Piece[3][3]
		tmpPiece[1][2] = Piece[2][3]
	end
	
	tmpPiece[2][2] = Piece[2][2]
	
	Piece = tmpPiece
end



function canMove(xVec,yVec)
	local xx = PieceX + xVec
	local yy = PieceY + yVec

	for x = 1, 3 do
		for y = 1, 3 do
			if ( Grid[xx + x][yy + y] and Piece[x][y]) then return false end
		end
	end
	return true
end



function newPiece()

	PieceX = 16
	PieceY = 6
	
	LastShape = math.random(10)
	if LastShape == 1 then Piece = { {true,false,true}, {false,true,false}, {true,false,true} } 		--X
	elseif LastShape == 2 then Piece = { {false,true,false}, {true,true,true}, {false,true,false} } 	--+
	elseif LastShape == 3 then Piece = { {true,false,false}, {true,true,true}, {false,false,true} } 	--S
	elseif LastShape == 4 then Piece = { {true,true,true}, {true,false,true}, {true,false,true} } 		--U
	elseif LastShape == 5 then Piece = { {true,true,true}, {true,false,true}, {true,false,false} } 		--P
	elseif LastShape == 6 then Piece = { {true,true,true}, {false,true,false}, {false,true,false} } 	--T
	elseif LastShape == 7 then Piece = { {true,true,false}, {true,false,false}, {true,false,false} } 	--1
	elseif LastShape == 8 then Piece = { {true,true,false}, {true,false,false}, {true,false,false} }
	elseif LastShape == 9 then Piece = { {false,true,false}, {false,true,false}, {false,true,false} } 	---
	elseif LastShape == 10 then Piece = { {false,true,false}, {false,true,false}, {false,true,false} } end 
end



function hitScan(centerX, centerY)
	local hitTableX = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
	local hitTableY = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
	local hitCounter = 0
	
	for x = -3,3 do
		for y = -3,3 do
			local xx = math.max(centerX + x, 2)
			xx = math.min(xx, 15)
			local yy = math.max(centerY + y, 2)
			yy = math.min(yy, 14)
			
			if zoneScan(xx, yy) then
				hitTableX[hitCounter] = xx
				hitTableY[hitCounter] = yy
				hitCounter = hitCounter + 1
			end
		end
	end
	
	if hitCounter > 0 then
		for i = 0, hitCounter-1 do
			zoneClear(hitTableX[i], hitTableY[i])
			Score = Score + 9 * Multiplier
			TimeSinceHit = 0.0
			Multiplier = 10
		end
	end
end



function zoneClear(centerX, centerY)
	Grid[centerX-1][centerY-1]	= false
	Grid[centerX-1][centerY]	= false
	Grid[centerX-1][centerY+1]	= false
	
	Grid[centerX][centerY-1]	= false
	Grid[centerX][centerY]		= false
	Grid[centerX][centerY+1]	= false
	
	Grid[centerX+1][centerY-1]	= false
	Grid[centerX+1][centerY]	= false
	Grid[centerX+1][centerY+1]	= false
	love.audio.stop(bip)
	love.audio.play(bip)
end



function zoneScan(centerX, centerY)
	local hit = true
	hit = hit and Grid[centerX-1][centerY-1]
	hit = hit and Grid[centerX-1][centerY]
	hit = hit and Grid[centerX-1][centerY+1]
	
	hit = hit and Grid[centerX][centerY-1]
	hit = hit and Grid[centerX][centerY]
	hit = hit and Grid[centerX][centerY+1]
	
	hit = hit and Grid[centerX+1][centerY-1]
	hit = hit and Grid[centerX+1][centerY]
	hit = hit and Grid[centerX+1][centerY+1]
	return hit
end


